// $LastChangedDate: 2009-05-15 14:25:58 -0500 (Fri, 15 May 2009) $

/*****************************************************************************
 * Compute diffuse and specular factors.
 * Returns total light intensity. 
 *****************************************************************************/
float
ComputeDiffuseSpecular( const vec4 ecPosition,
                        const float diffuseScale,
                        const float specularScale,
                        const float specularExponent )
{
//  const vec4 ecLightPos  = gl_ModelViewMatrix * uni_light0_position;  // light position
    const vec4 ecLightPos  = uni_light0_position;
    const vec3 ecNormal    = normalize( gl_NormalMatrix * gl_Normal );  // surface normal
    const vec4 ecLight     = normalize( ecLightPos - ecPosition );      // light vector
    const vec3 ecReflect   = reflect( -ecLight, ecNormal );             // reflected light vector from surface
    const vec4 ecView      = normalize( -ecPosition );                  // viewpoint
    float diffuse = max( dot( ecLight, ecNormal ), 0.0 );
    // No specular if surface normal doesn't face VP.
    if ( diffuse > 0.0 )
    {
        // Facing.
        float specular = max( dot( ecReflect, ecView ), 0.0 );
        specular = pow( specular, specularExponent );
        return diffuse * diffuseScale + specular * specularScale;
    }
    else
    {
        // Not facing.
        return diffuse * diffuseScale;
    }
}
