/*****************************************************************************
 * $LastChangedDate: 2009-05-13 07:46:03 -0500 (Wed, 13 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Module configuration.
 *//*
 * LEGAL:	COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define GRAPH_CONF_CC 1
#include "base/module.hh"
#include "base/conf.hh"
#include "base/misc.hh"
using namespace base;
#include "gfx/rgba.hh"
using namespace gfx;
#include "graph/module.hh"
#include "graph/defs.hh"
#include "graph/conf.hh"

namespace graph {

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////  Conf  ///////////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

/*****************************************************************************
 * ctor/dtor.
 *****************************************************************************/
Conf::Conf( void )
:	mValid(false),
	// cmd-line args
	mEnableOsgOptimizer(true),
	mEnableFog(true),
	// .conf file
	// Environment:
	mBackgroundColor(graph::defs::DEFAULT_BACKGROUND_COLOR)
{
	// NOP
}

/*****************************************************************************
 * Initialize/read configuration.
 *****************************************************************************/
void
Conf::Init( int argc, char** argv, const string& confTxtFilename )
{

BEGIN_ONCE {

	mValid = true;  // assign this first as *_CONF may expand to GetConf() which asserts it

	ParseCmdlineArgs( argc, argv );   // cmdline args parsed prior to reading .conf file
	ReadConfTxt( confTxtFilename );   // read from textual .conf file
	ParseCmdlineArgs2( argc, argv );  // cmdline args that override .conf file

} END_ONCE

}

/*****************************************************************************
 * Parse cmdline args.
 *****************************************************************************/
void
Conf::ParseCmdlineArgs( int argc, char** argv )
{
	// NOP
}

void
Conf::ParseCmdlineArgs2( int argc, char** argv )
{
	for ( int i = 1; i < argc; ++i )
	{
		if ( STREQ( argv[i], "-no-osg-opt" ) )
		{
			mEnableOsgOptimizer = false;
			CmdargDone( i );
			CLOG << "Disabled OSG optimizer." << endl;
		}
	}
}

/*****************************************************************************
 * Read from .conf file.
 *****************************************************************************/
void
Conf::ReadConfTxt( const string& confTxtFilename )
{
	try
	{
		ModuleConfTxt confTxt;
		confTxt.Read( string(base::GetDataDir()) + string(confTxtFilename) );
	}
	catch (...)
	{
		CERROR << "ERROR: cannot read " << confTxtFilename << endl;
		EXIT( 1 );
	}
}

void
Conf::ModuleConfTxt::ProcessField( string& field, ifstream& conf )
{
	bool error = false;

	if ( field == "BackgroundColor" )
	{
		fp r, g, b, a;
		conf >> r;
		conf >> g;
		conf >> b;
		conf >> a;
		GRAPH_CONF.mBackgroundColor.Set( r, g, b, a );
	}

    // Errors?
	if ( error )
	{
		CERROR << "ERROR[conf]: parameter for " << field << " is invalid." << endl;
		EXIT( 1 );
	}
}

/*****************************************************************************
 * Print Conf configuration.
 *****************************************************************************/
ostream&
operator<<( ostream& strm, const Conf& conf )
{
	std::ios::fmtflags savedFlags = strm.flags();

	strm << "conf [graph]:"                                                      << endl
	     << "enable OSG optimizer        = " << conf.mEnableOsgOptimizer      ;//<< endl
	strm.flags(savedFlags);

	return strm;
}

} // namespace graph
