/*****************************************************************************
 * $LastChangedDate: 2009-05-07 14:43:02 -0500 (Thu, 07 May 2009) $
 * @file
 * @author  Jim E. Brooks  http://www.palomino3d.org
 * @brief   Interface of C functions for Lua scripts.
 * @remarks @remarks module.txt documents how to export new C++ classes to Lua.
 *
 * ----------------------------
 * -- THIS IS AUTO-GENERATED --
 * ----------------------------
 *
 *//*
 * LEGAL:   COPYRIGHT (C) 2008 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#include "base/module.hh"
using namespace base;
#include "shader/module.hh"
#include "shader/shader_settings.hh"
using namespace shader;
#include "sky/module.hh"
using namespace sky;
#include "lua_bind/module.hh"
#include "lua_bind/lua_lang.hh"
#include "lua_bind/bind.hh"

namespace lua_bind {

#define BIND_CLASS_NAME      "Sky"
#define BIND_METATABLE_NAME  LUABIND_METATABLE_STRING(BIND_CLASS_NAME)

//==============================================================================
// C functions called by Lua that access C++ objects.
//==============================================================================

/*****************************************************************************
 * Sky:SetLightColor( ambient:RGBA, diffuse:RGBA, specular:RGBA )
 *****************************************************************************/
INTERN int 
Sky_SetLightColor( lua_State* L ) 
{ 
   try { 

		// Historical: Sky used to have the hw light.
		const RGBA specular( POP_RGBA( L ) );
		const RGBA diffuse(  POP_RGBA( L ) );
		const RGBA ambient(  POP_RGBA( L ) );
		GET_SHADER_SETTINGS().SetLightColors( 0, ambient, diffuse, specular );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

/*****************************************************************************
 * Sky:SetLightPosition( Vec3 )
 *****************************************************************************/
INTERN int 
Sky_SetLightPosition( lua_State* L ) 
{ 
   try { 

		// Historical: Sky used to have the hw light.
		const WorldVertex pos = POP_VECTOR3<WorldVertex>( L );
		GET_SHADER_SETTINGS().SetLightPosition( 0, pos );
    
   } CATCH_LUA_EXCEPTION 
   return 0; 
} 

//==============================================================================
// Register C functions with Lua. 
//==============================================================================

#if ! DOXYGEN 

INTERN const struct luaL_reg sMethodTableNew[] =
{
  // new,__gc omitted as Lua cannot instantiate this C++ class.
  { NULL, NULL }
};

INTERN const struct luaL_reg sMethodTable[] =
{
  // __gc omitted as Lua cannot instantiate this C++ class.
  { "SetLightColor", Sky_SetLightColor },
  { "SetLightPosition", Sky_SetLightPosition },
  { NULL, NULL }
};

void
RegisterSky( lua_State* L )
{
	REGISTER_CLASS_METATABLE( L,
                              BIND_METATABLE_NAME,
                              BIND_CLASS_NAME,
                              sMethodTableNew,
                              sMethodTable );
}

#endif // ! DOXYGEN 

} // namespace lua_bind 
