/*****************************************************************************
 * $LastChangedDate: 2009-05-11 18:50:07 -0500 (Mon, 11 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Hardware light for 3D models incompatible with shaders.
 *//*
 * LEGAL:   COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#if COMPILE_HW_LIGHT
#ifndef SHADER_HW_LIGHT_HH
#define SHADER_HW_LIGHT_HH 1

#define GET_HW_LIGHT() (HwLight::GetInstance())

#include <osg/Light>
#include <osg/LightSource>
#include <osg/Fog>

namespace shader {

////////////////////////////////////////////////////////////////////////////////
/// @brief The hardware light for 3D models incompatible with shaders.
///
/// Simulates the sun.  Also includes hw fog.
///
class HwLight
{
PREVENT_COPYING(HwLight)
private:
	HwLight( void );
	~HwLight();

public:
    CLASS_METHOD HwLight& GetInstance( void )  // Singleton
	{
		if ( UX( msInstance == NULL ) )
			msInstance = new HwLight;
		return *msInstance;
	}

	RefPtr<osg::LightSource>	MakeHwLightGroup( void );
    void                        SetBrightness( const fp brightness );
    void                        SetAmbient( const RGBA ambient );
    void                        SetDiffuse( const RGBA diffuse );
    void                        SetSpecular( const RGBA specular );
    enum                        { LIGHT_DIRECTIONAL = true, LIGHT_POSITIONAL = false };
    void                        SetPosition( const WorldVertex position, const bool directional );
    void                        SetFogColor( const RGBA fogColor );
    void                        SetFogMode( const osg::Fog::Mode mode );
    void                        SetFogDensity( const fpx density );

private:
    CLASS_CONST uint    HW_LIGHT_NUM = 0;   ///< GL_LIGHT0 is wrong in OSG
	CLASS_VAR HwLight*  msInstance;
	RefPtr<osg::Light>	mHwLight;           ///< hw light
    RefPtr<osg::Fog>    mHwFog;             ///< hw fog
    osg::Vec4           mAmbient;
    osg::Vec4           mDiffuse;
    osg::Vec4           mSpecular;
    osg::Vec4           mFogColor;
};

} // namespace shader

#endif // SHADER_HW_LIGHT_HH
#endif // COMPILE_HW_LIGHT
