/*****************************************************************************
 * $LastChangedDate: 2009-05-11 18:50:07 -0500 (Mon, 11 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Module.
 *//*
 * LEGAL:	COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define SHADER_MODULE_CC 1
#include "base/module.hh"
using namespace base;
#include "graph/module.hh"
using namespace graph;
#include "shader/module.hh"
#include "shader/conf.hh"
#include "shader/shader_args.hh"
#include "shader/shader_funcs.hh"
#include "shader/shader_settings.hh"
#include "shader/shader_name.hh"

namespace shader {

/*****************************************************************************
 * Initialize module.
 *****************************************************************************/
void
Init( const InitArgs& initArgs )
{
	// Load textual conf file into Conf struct.
	const string confTxtFilename = GetConfTxtFilename( initArgs.mProgramName );
	// Not CONF macro since mValid isn't true yet.
	shader::global::gConf.Init( initArgs.mArgc, initArgs.mArgv, confTxtFilename );
}

/*****************************************************************************
 * Start module.
 *****************************************************************************/
void
Start( void )
{
	RefPtr<GroupNode> rootNode = GET_SCENE_GRAPH().GetRootNode();
    ShaderSettings& shaderSettings = GET_SHADER_SETTINGS();

    // Shader uniform defaults.
	ResetShaderUniformsForRootNode( *rootNode );

    // Light #0 position.
    shaderSettings.SetLightPosition( 0, SHADER_CONF.mLight0Position );

    // Fog.
	shaderSettings.SetFogColor( SHADER_CONF.mFogColor );
	shaderSettings.SetFogDensity( SHADER_CONF.mFogDensity );

    // Brightness.
    shaderSettings.SetBrightness( SHADER_CONF.mBrightness );
}

} // namespace shader
