/*****************************************************************************
 * $LastChangedDate: 2009-05-15 14:25:58 -0500 (Fri, 15 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Shader name.
 *//*
 * LEGAL:   COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef SHADER_SHADER_NAME_HH
#define SHADER_SHADER_NAME_HH 1

#include "shader/defs.hh"

namespace shader {

//------------------------------------------------------------------------------
// Specific names:
#define UNIFORM_NAME_SHADER_MODE_TEXTURE  "uni_shaderMode_texture"
#define UNIFORM_NAME_SHADOW_MODE          "uni_shadowMode"
#define UNIFORM_NAME_GLASS                "uni_glass"
#define UNIFORM_NAME_ALPHA_TEX            "uni_alphaTex"
#define UNIFORM_NAME_ALPHA_BIAS           "uni_alphaBias"
#define UNIFORM_NAME_TEXTURE_0            "uni_texture_0"
#define UNIFORM_NAME_TEXTURE_SHADOW       "uni_texture_shadow"
#define UNIFORM_NAME_BRIGHTNESS           "uni_brightness"
//------------------------------------------------------------------------------

////////////////////////////////////////////////////////////////////////////////
/// @brief ID of a shader.
///
/// Default shader can be referenced by either Shader("default") or Shader().
/// Rationale is that Lua scripts could use "default" or omit name.
///
class ShaderName
{
public:
	ShaderName( void )
	: mName(SHADER_NAME_DEFAULT), mDefault(true) { }

	ShaderName( const string& name )
	: mName(name.empty() ? SHADER_NAME_DEFAULT : name),
      mDefault(mName == SHADER_NAME_DEFAULT)  // --ORDER-DEPENDENT-- in case default name passed as string
    { }

	bool			IfDefaultShader( void ) const { return mDefault; }
	const string&	GetString( void ) const { return mName; }

	friend bool operator<( const ShaderName& a, const ShaderName& b ) { return a.mName < b.mName; }
	friend bool operator==( const ShaderName& a, const ShaderName& b ) { return a.mName == b.mName; }
	       bool operator==( const char* name ) const { return mName == name; }

private:
	const string	mName;
	const bool		mDefault;	///< if name is of the default shader (no need to be tested by operator<())
};

} // namespace shader

#endif // SHADER_SHADER_NAME_HH
