/*****************************************************************************
 * $LastChangedDate: 2009-05-11 18:50:07 -0500 (Mon, 11 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Interface to shader settings.
 *//*
 * LEGAL:	COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#define SHADER_SHADER_SETTINGS_CC 1
#include "base/module.hh"
using namespace base;
#include "graph/module.hh"
#include "graph/scene_graph.hh"
using namespace graph;
#include "shader/module.hh"
#include "shader/conf.hh"
#include "shader/shader_args.hh"
#include "shader/shader_settings.hh"
#if COMPILE_HW_LIGHT
#include "shader/hw_light.hh"
#endif

namespace shader {

////////////////////////////////////////////////////////////////////////////////
/////////////////////////////  ShaderSettings  /////////////////////////////////
////////////////////////////////////////////////////////////////////////////////

ShaderSettings* ShaderSettings::msInstance;

/*****************************************************************************
 * Set position of a light source.
 *****************************************************************************/
void
ShaderSettings::SetLightPosition( const uint lightNum, const osg::Vec3& pos )
{
ASSERT( lightNum == 0 );  // GLSL shader src only supports one light position

	SetUniform<osg::Vec4>( "uni_light0_position",
                           // w==0 directional light
                           osg::Vec4( pos[XX], pos[YY], pos[ZZ], 0.0f ),
                           *GET_SCENE_GRAPH().GetRootNode() );
}

/*****************************************************************************
 * Set colors of a light source.
 *****************************************************************************/
void
ShaderSettings::SetLightColors( const uint lightNum, const RGBA ambient, const RGBA diffuse, const RGBA specular )
{
//DEBUG_CODE( CDEBUG << "UNIMPLEMENTED ShaderSettings::SetLightColors()" << endl; )
}

/*****************************************************************************
 * Set fog density.
 *****************************************************************************/
void
ShaderSettings::SetFogDensity( const fpx fogDensity_, const osg::Fog::Mode fogMode )
{
    const fpx fogDensity = SHADER_CONF.mEnableFog ? fogDensity_ : 0.0;

    // Shader:
	SetUniform<float>( "uni_fogDensity", fogDensity, *GET_SCENE_GRAPH().GetRootNode() );

#if COMPILE_HW_LIGHT
    // Hardware light:
    GET_HW_LIGHT().SetFogDensity( fogDensity );
    GET_HW_LIGHT().SetFogMode( fogMode );
#endif
}

/*****************************************************************************
 * Set fog color.
 *****************************************************************************/
void
ShaderSettings::SetFogColor( const RGBA fogColor )
{
    // Shader:
	SetUniform<osg::Vec4>( "uni_fogColor", RGBA2VEC4F(fogColor), *GET_SCENE_GRAPH().GetRootNode() );  

#if COMPILE_HW_LIGHT
    // Hardware light:
    GET_HW_LIGHT().SetFogColor( fogColor );
#endif
}

/*****************************************************************************
 * Set brightness.
 *****************************************************************************/
void
ShaderSettings::SetBrightness( const fp brightness )
{
    mBrightness = brightness;

    // Shader:
	SetUniform<float>( "uni_brightness", brightness, *GET_SCENE_GRAPH().GetRootNode() );

#if COMPILE_HW_LIGHT
    // Hardware light:
    GET_HW_LIGHT().SetBrightness( brightness );
#endif
}

/*****************************************************************************
 * Get brightness.
 *****************************************************************************/
fp
ShaderSettings::GetBrightness( void ) const
{
    return mBrightness;
}

} // namespace shader
