/*****************************************************************************
 * $LastChangedDate: 2009-05-11 18:50:07 -0500 (Mon, 11 May 2009) $
 * @file
 * @author	Jim E. Brooks  http://www.palomino3d.org
 * @brief	Interface to shader settings.
 *//*
 * LEGAL:   COPYRIGHT (C) 2009 JIM E. BROOKS
 *          THIS SOURCE CODE IS RELEASED UNDER THE TERMS
 *          OF THE GNU GENERAL PUBLIC LICENSE VERSION 2 (GPL 2).
 *****************************************************************************/

#ifndef SHADER_SHADER_SETTINGS_HH
#define SHADER_SHADER_SETTINGS_HH 1

#include <osg/Fog>
#include "gfx/osg.hh"
#include "gfx/rgba.hh"
using namespace gfx;

namespace shader {

#define GET_SHADER_SETTINGS() (ShaderSettings::GetInstance())

////////////////////////////////////////////////////////////////////////////////
/// @brief Interface to shader settings (hides specific uniform names).
///
/// Provides a traditional OpenGL interface over shaders
/// to shader-emulated light sources, fog, etc.
///
class ShaderSettings
{
PREVENT_COPYING(ShaderSettings)
private:
	ShaderSettings( void ) : mBrightness(1.0f) { }
	~ShaderSettings() { }

public:
    CLASS_METHOD ShaderSettings& GetInstance( void )  // Singleton
	{
		if ( UX( msInstance == NULL ) )
			msInstance = new ShaderSettings;
		return *msInstance;
	}

	void	SetLightPosition( const uint lightNum, const osg::Vec3& pos );
    void    SetLightColors( const uint lightNum, const RGBA ambient, const RGBA diffuse, const RGBA specular );
    void    SetFogDensity( const fpx fogDensity, const osg::Fog::Mode fogMode = osg::Fog::EXP );
    void    SetFogColor( const RGBA fogColor );
    void    SetBrightness( const fp brightness );
    fp      GetBrightness( void ) const;

private:
	CLASS_VAR ShaderSettings*   msInstance;
    fp                          mBrightness;
};

} // namespace shader

#endif // SHADER_SHADER_SETTINGS_HH
